#!/usr/local/bin/python3
# -*- coding: utf-8 -*-

# importera_gedcom_tråd.py

# Senast ändrad: 2010-12-31  - Uppdateras automatiskt. Ändra inte manuellt.
###########################################################################
#    Copyright (c) 2008, 2011 S Ingemar Johansson
#    ingemar at sijohansson dot com
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License, version 3,
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################




########################################################################################
import sys
import os
import stat

from johanklasser import johanpersonfil, johanparfil
from johangemensamt import allmänt
from PyQt4.QtCore import *
from PyQt4.QtGui import *

########################################################################################




########################################################################################
class Importera_Gedcom_Tråd(QThread):

    SenastÄndrad = '2010-12-31 10:01:35'  # Uppdateras automatiskt. Ändra inte manuellt.

    loggfilfolder = ""
    loggfilgrundnamn = 'importera_gedcom_py3_'
    loggfilopen = False

    filattöppna = ""
    nydatabas = ""
    importfil = ""

    redanlästrad = ""
    gedcomfil = ""

    senastesignal = ""
    senastenivå = ""

    EndOfFile = False

    personlista_johan = []
    personlista_gedcom = []
    parlista_johan = []
    parlista_gedcom = []

    rader=[]

########################################################################################
    def __init__(self, lock, parent=None):
        super(Importera_Gedcom_Tråd, self).__init__(parent)
        self.lock = lock
        self.stopped = False
        self.mutex = QMutex()
        self.path = None
        self.completed = False



        sysplatform = sys.platform
        if sysplatform == "win32":
            try:
                self.loggfilfolder =  os.path.join( os.environ['APPDATA'] , 'sij', 'johanloggar' )
            except:
                pass
        if sysplatform == "linux2":
            try:
                self.loggfilfolder = os.path.join( os.environ['HOME'], 'johanloggar' )
            except:
                pass


        for i in range(0,20):
            self.rader.append("*")

        return
########################################################################################




###################################################################################################
    def __del__(self):

        if self.loggfilopen == True:
            self.logga ('tråd: Stänger logg för ImporteraGedcom_py3\n')
            self.loggfilopen = False
            self.loggfil.close()

        return
###################################################################################################




########################################################################################
    def initialize(self,fil2open, nydb, imprt):
        self.filattöppna = fil2open

        self.logga("tråd: Fil att öppna: " + str(self.filattöppna) + '\n')
        print("tråd: Fil att öppna: " + str(self.filattöppna))

        self.nydatabas = nydb
        print("tråd: Ny databas att importera till: " + str(self.nydatabas))

        self.importfil = imprt
        print("tråd: Gedcomfil att importera: " + str(self.importfil))


        self.rader=[]
        for i in range(0,20):
            self.rader.append("*")

        return
########################################################################################




########################################################################################
    def stop(self):
        try:
            self.mutex.lock()
            self.stopped = True
        finally:
            self.mutex.unlock()
########################################################################################




########################################################################################
    def isStopped(self):
        try:
            self.mutex.lock()
            return self.stopped
        finally:
            self.mutex.unlock()
########################################################################################




########################################################################################
    def run(self):
        self.importera_gedcom_tråd()
        self.stop()
        self.emit(SIGNAL("finished(bool)"), self.completed)
########################################################################################




########################################################################################
    def importera_gedcom_tråd(self):

# *** skapa listor *********************************************************************
# listor med person- och parnummer i gedcom-filen
#
#        self.logga ("Öppnar för läsning: " + self.filattöppna + '\n')
#        self.logga ('Genomsöker filen efter "INDI", "FAM", "CHIL", "FAMC", "FAMS" \n')


# Filen öppnas här för läsning och ska ha kodning utf-8.
# Nödvändig konvertering från eventuell ANSEL- eller annan kodning ska vara gjorde tidigare.

        self.gedcomfil = open( self.filattöppna, "r" )
        while True:
            if self.isStopped():
                return

            rad = self.gedcomfil.readline()
            if rad == '':
                self.gedcomfil.close()
                break

            raddelar = rad.strip().split(' ')

            if len(raddelar) >2:
                if raddelar[0] == "0":
                    gedcomnummer= raddelar[1].strip('@')
                    if len(raddelar[1]) == len(gedcomnummer) + 2:  # kolla ATT EXAKT TVÅ TECKEN tagits bort
                        if raddelar[2] == "INDI":
                            if self.personlista_gedcom.count(gedcomnummer) == 0:
                                self.personlista_gedcom.append(gedcomnummer)
                        if raddelar[2] == "FAM":
                            if self.parlista_gedcom.count(gedcomnummer) == 0:
                                self.parlista_gedcom.append(gedcomnummer)



                if raddelar[0] == "1":
                    gedcomnummer= raddelar[2].strip('@')
                    if len(raddelar[2]) == len(gedcomnummer) + 2:  # kolla ATT EXAKT TVÅ TECKEN tagits bort
                        if raddelar[1] == "CHIL":
                            if self.personlista_gedcom.count(gedcomnummer) == 0:
                                self.personlista_gedcom.append(gedcomnummer)
                        if raddelar[1] == "FAMC"  or raddelar[1] == "FAMS" :
                            if self.parlista_gedcom.count(gedcomnummer) == 0:
                                self.parlista_gedcom.append(gedcomnummer)


# *** listor klara *********************************************************************




# *** sortera **************************************************************************

# Om inkommande GEDCOM-fil är ett utdrag ur en johandatabas och innehåller
# johannumrerade person- och parnummer, ska man försöka behålla dem?
# Svar: NEJ, tills vidare inte!
#
# Det kan möjligen se snyggt ut vid testkörningar av export/import i Johan.
# I verklig användning behöver man inte gå vägen över gedcom för att importera
# ett johanutdrag till en annan johandatabas.
#
#
# INDI- och FAM-beteckningar i inkommande gedcom behöver i princip inte
# innehålla någon numerisk del i decimal form (men gör det nog vanligen).
# I den mån de innehåller en numerisk del behöver denna del från de olika
# personerna (paren) inte bilda en vacker serie från ett och uppåt med
# inkrementet 1 (och gör det troligen sällan)
#

# Sortera listorna från gedcomfilen? - OK!
# Jag ser inte någon teknisk anledning att göra det, men det kan kanske
# se snyggt ut att personer respektive par i importdatabasen kommer i
# samma ordning som de hade i den exporterande databasen, även om inte
# nummerdelarna blir lika. Sortering är enkelt gjord:
        self.personlista_gedcom.sort()
        self.parlista_gedcom.sort()

#        print ""
#        print "INDI-beteckningar i importfilen: ", self.personlista_gedcom
#        print ""
#        print "FAM-beteckningar i importfilen:  ", self.parlista_gedcom
#        print ""

#
# Fundera en annan gång över fallet att värdena i personlista och parlista har
# en numerisk del som är unik för respektive person och respektive par.
# I så fall kanske man kan extrahera denna numeriska del och göra om den till johannummer.
#
# Det skulle i så fall även innebära att en gedcomfil som exporterades från en johandatabas
# i den importerade databasen får behålla de ursprunliga person- och parnumren.
# *** sortering klar *******************************************************************




# *** skapa listor med person- och parnummer för johan-databasen ***********************
        nr = 0
        for i in range(0,len (self.personlista_gedcom)):
            nr=nr+1
            self.personlista_johan.append(str(nr).zfill(8))

        senastepersonnummer = nr
#        self.logga ("Skapade johan-personnummer " + str("1").zfill(8) + " till " +  str(nr).zfill(8) + '\n')

        nr = 0
        for i in range(0,len (self.parlista_gedcom)):
            nr=nr+1
            self.parlista_johan.append("par" + str(nr).zfill(8))
#        self.logga ("Skapade johan-parnummer par" + str("1").zfill(8) + " till par" +  str(nr).zfill(8) + '\n')

        senasteparnummer = nr
# *** listor klara *********************************************************************




# *** skapa johandatabasens katalogstomme **********************************************

        allmänt.skapanydatabas(allmänt(), self.nydatabas)
#        self.logga ("Skapade tom databas" + str(self.nydatabas) + '\n')

        senasteperson = str(senastepersonnummer).zfill(8)
        senastepar = 'par' + str(senasteparnummer).zfill(8)

        senastepersonfilnamn = os.path.join(str(self.nydatabas), "senaste", "senasteperson.txt")
        senasteparfilnamn = os.path.join(str(self.nydatabas), "senaste", "senastepar.txt")

        senaste = open(senastepersonfilnamn, "w")
        senaste.write(senasteperson)
        senaste.close()
#        self.logga ('Skrev i förväg ' + senasteperson + ' i ' + senastepersonfilnamn + '\n')

        senaste = open(senasteparfilnamn, "w")
        senaste.write(senastepar)
        senaste.close()
#        self.logga ('Skrev i förväg ' + senastepar + ' i ' + senasteparfilnamn + '\n')
# *** johandatabasens stomme klar *****************************************************




# *** fyll i johandatabasen *************************************************************
        self.gedcomfil = open( self.filattöppna, "r" )  # Filen öppnas igen för läsning
#        self.logga ("Öppnar för läsning: " + self.filattöppna + '\n')

        self.redanlästrad = "*"      # rensar inför första anropet till self.gemignästarad()
        self.gemignästarad()            # levererar utf-8 i globala "self.rader[]" !
        while True:

#            self.logga("Hämtat ett paket:" + '\n')
#            for rad in self.rader:
#                self.logga(rad + '\n')


            rad0delar = self.rader[0].strip().split(' ')

            if self.EndOfFile:
#                self.logga ( 'End Of File - i "Fyll i johandatabasen" -1\n')
                break

            if len(rad0delar) > 2:
                if rad0delar[2] == "INDI":
                    self.personblad()
                elif rad0delar[2] == "FAM":
                    self.parblad()

            if self.EndOfFile:
#                self.logga ( 'End Of File - i "Fyll i johandatabasen" -2\n')
                break

            self.gemignästarad()   # levererar utf-8 i globala "self.rader[]" !


        self.gedcomfil.close()
#        self.logga ("Stängde : " + self.filattöppna + '\n')

# **************************************************************************************




# *** Fullständiga par *****************************************************************
# Nackdel som skulle kunna undvikas: Holger gör det svårt att indikera olika fäder till
# en moders två barn, båda med okänd fader.
#
# Gedcom tillåter i och för sig FAM-poster med enbart ena föräldern och inte den andra.
#
# I Johan skapas som första alternativ för varje barn med okända andra förälder separat
# par-fil och dessutom en (tom) personfil för den andra föräldern.
#
# Importerade par-filer med enbart en förälder kompletteras här med även den saknade föräldern
# (trots att all information om denne ännu saknas)

        Person1 = johanpersonfil()
        Par1 = johanparfil()

#        self.logga  ('Kompletterar halva föräldrapar med okänd förälder\n')
#        self.logga ( "Nya personer: \n")

        allmänt.initpersonfil(allmänt(), Person1)
        for par in self.parlista_johan:
            parfilnamn = os.path.join( self.nydatabas, "data", par[0:8], par + ".xml")
            allmänt.hämtaparfil(allmänt(), parfilnamn, Par1)
            if Par1.person1_ID != ""  and Par1.person2_ID == "":   # kvinnan saknas (person2)
                senastepersonnummer = senastepersonnummer + 1
                senasteperson = str(senastepersonnummer).zfill(8)
                self.emit(SIGNAL("indexed(QString)"), senasteperson)

                personfilnamn = os.path.join(str(self.nydatabas), "data", senasteperson[0:5], senasteperson + ".xml")
                Par1.person2_ID = senasteperson
                Person1.person_ID = senasteperson
                Person1.egna_par_ID = Par1.par_ID
                Person1.person_kön = "k"
                allmänt.sparaparfil(allmänt(), parfilnamn, Par1)
                allmänt.sparapersonfil(allmänt(), personfilnamn, Person1)
#                self.logga ( ",  k" + str(senasteperson) + u'\n')

            elif Par1.person2_ID != ""  and Par1.person1_ID == "":   # mannen saknas (person1)
                senastepersonnummer = senastepersonnummer + 1
                senasteperson = str(senastepersonnummer).zfill(8)
                self.emit(SIGNAL("indexed(QString)"), senasteperson)

                personfilnamn = os.path.join(str(self.nydatabas), "data", senasteperson[0:5], senasteperson + ".xml")
                Par1.person1_ID = senasteperson
                Person1.person_ID = senasteperson
                Person1.egna_par_ID = Par1.par_ID
                Person1.person_kön = "m"
                allmänt.sparaparfil(allmänt(), parfilnamn, Par1)
                allmänt.sparapersonfil(allmänt(), personfilnamn, Person1)
#                self.logga (",  m" + str(senasteperson) + u'\n')
#        self.logga ("\n")

        senaste = open(senastepersonfilnamn, "w")
        senaste.write(senasteperson)
        senaste.close()

#        self.logga ("Alla data inlästa.\n")
#        self.logga ('Skrev nytt värde, ' + senasteperson + ',  i ' + senastepersonfilnamn + '\n')




# *** rensa databasen *****************************************************************
#
# Efter avslutad import bör i datakatalogerna raderas
#   1) alla *.xml.?  men inte *.xml
#   2) alla *.txt med längden =0

        self.emit(SIGNAL("indexed(QString)"), "Rensar temporära filer")

#        self.logga ('Rensar temporära filer\n')
        for f in os.listdir(os.path.join(self.nydatabas, "data")):
            if os.path.isdir(os.path.join(self.nydatabas, "data", f)):
                for fil in os.listdir(os.path.join(self.nydatabas, "data", f)):
                    filsökväg = os.path.join(self.nydatabas, "data", f, fil)
                    if os.path.isfile(filsökväg):
                        filnamn = os.path.basename(filsökväg).split('.')
                        if len(filnamn) == 3:
                            if filnamn[2].isdigit() and filnamn[1]=='xml' :    # om filnam = *.xml.?  där ? är ett tal
                                os.remove(filsökväg)                            # så radera filen
                        elif len(filnamn) == 2:
                            if filnamn[1]== 'txt':                             # om filnamn = *.txt
                                if int(os.stat(filsökväg)[stat.ST_SIZE]) == 0:  # och filstorleken == 0
                                    os.remove(filsökväg)                        # så radera filen


        self.emit(SIGNAL("indexed(QString)"), "Importen är klar")

        return
###################################################################################################




###################################################################################################
    def personblad(self):

        månader = ['   ', 'JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC']

        rad0delar = self.rader[0].strip().split(' ')
        rad1delar = self.rader[1].strip().split(' ')
        rad2delar = self.rader[2].strip().split(' ')
        rad3delar = self.rader[3].strip().split(' ')
#        rad4delar = self.rader[4].strip().split(' ')
#        rad5delar = self.rader[5].strip().split(' ')
#        rad6delar = self.rader[6].strip().split(' ')
#        rad7delar = self.rader[7].strip().split(' ')
#        rad8delar = self.rader[8].strip().split(' ')
#        rad9delar = self.rader[9].strip().split(' ')

        if rad1delar[0] == "*":
            return
        if len(rad1delar) < 2 :
            return


        person_gedcom = rad0delar[1].strip('@')
        personindex = self.personlista_gedcom.index(person_gedcom)
        person_johan = self.personlista_johan[personindex]             #      /ascii


        if self.senastesignal != person_johan:
            self.emit(SIGNAL("indexed(QString)"), person_johan)
            self.senastesignal = person_johan

        underkatalog = os.path.join(self.nydatabas, "data", person_johan[0:5])
        if not os.path.exists(underkatalog):
            os.mkdir(underkatalog)
        xmlfilnamn = os.path.join(underkatalog, person_johan + ".xml")
        txtfilnamn = os.path.join(underkatalog, person_johan + ".txt")

        txtfil = open(txtfilnamn, 'a')

        Person1 = johanpersonfil()

        allmänt.initpersonfil(allmänt(), Person1)


        if not os.path.exists(xmlfilnamn):
            Person1.person_ID = person_johan

            Person1.anteckning = 'INDI-nr ' + person_gedcom + ', från gedcom-fil ' + self.importfil
            allmänt.sparapersonfil(allmänt(), xmlfilnamn, Person1)

        allmänt.hämtapersonfil(allmänt(), xmlfilnamn, Person1)
        if Person1.person_ID == "":
           Person1.person_ID = person_johan


# -------------------------------------------------
        if rad1delar[1] == "NAME":

            pos = self.rader[1].index("NAME")
            namnrad = self.rader[1][pos+4:]
            if namnrad.count('/') == 2:
                pos1 = namnrad.index('/')
                pos2 = namnrad.index('/', pos1+1)
                förnamnrad = namnrad[0:pos1] + namnrad[pos2+1:]           # både delen för efternamn och delen efter efternamn
                efternamnrad = namnrad[pos1+1:pos2]
            elif namnrad.count('/') < 2:
                förnamnrad=namnrad
                efternamnrad = ""
            else:
                förnamnrad=namnrad
                efternamnrad = ""
                txtfil.write( self.rader[1] + '\n' )

            fornam = förnamnrad.split(' ')
            for nam in fornam:

                Person1.förnamn = self.adderanamn( Person1.förnamn, nam.strip())

            efternam = efternamnrad.split(' ')
            for nam in efternam:

                Person1.efternamn = self.adderanamn( Person1.efternamn , nam.strip())

            if self.rader[2] != '*':

                if rad2delar[1] == "GIVN":
                    pos = self.rader[2].index("GIVN")
                    namnrad = self.rader[2][pos+4:]
                    fornam = namnrad.split(' ')
                    for nam in fornam:

                        Person1.förnamn = self.adderanamn( Person1.förnamn, nam.strip())
                    self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3

                elif rad2delar[1] == "SURN":
                    pos = self.rader[2].index("SURN")
                    namnrad = self.rader[2][pos+4:]
                    efternam = namnrad.split(' ')
                    for nam in efternam:

                        Person1.efternamn = self.adderanamn( Person1.efternamn, nam.strip())
                    self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3

                elif rad2delar[1] == "NICK":
                    pos = self.rader[2].index("NICK")
                    namnrad = self.rader[2][pos+4:]
                    tilltal = namnrad.split(' ')
                    for nam in tilltal:

                        Person1.tilltal = self.adderanamn( Person1.tilltal, nam.strip())
                    self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3

                else:
                    self.skrivöverblivet(txtfil, self.rader, 2) # om annat än GIVN, SURN, NICK på nivå 2


# -------------------------------------------------
# Är detta en rest från Holgers implementation ?
#        elif rad1delar[1] == "FORE":
#            Person1.förnamn = Person1.förnamn + rad1delar[2] + "\n"


# -------------------------------------------------
        elif rad1delar[1] == "SEX":

            if len(rad1delar) == 3:

                if rad1delar[2] == "F":
                    Person1.person_kön = "k"
                elif rad1delar[2] == "M":
                    Person1.person_kön = "m"
                else:
                    self.skrivöverblivet(txtfil, self.rader, 2) # om annat än F eller M på raden
            else:
                self.skrivöverblivet(txtfil, self.rader, 2)     # om mindre eller mera än F eller M på raden


# -------------------------------------------------
        elif  rad1delar[1] == "BIRT":

#            self.logga( 'In BIRT: len(rad1delar)= ' + str(len(rad1delar)) + '\n' )
            if len(rad1delar) == 2:

#                self.logga( 'In BIRT: len(rad2delar)= ' + str(len(rad2delar)) + '\n' )
                if len(rad2delar) > 2: # Om för kort rad så stupar man på nästa rad, om för lång: okänt innehåll

#                    self.logga( 'In BIRT: rad2delar[1]= ' + str(rad2delar[1]) + '\n' )
                    if rad2delar[1] == "DATE":
                        if len(rad2delar) > 5:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Född datum") # om mera än förväntat på DATE-raden
                        elif len(rad2delar) <3:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Född datum") # om mindre än förväntat på DATE-raden
                        else:
                            födddag   = ""
                            föddmånad = ""
                            föddår    = ""

                            if len(rad2delar) == 5:
                                födddag   = rad2delar[2]
                                föddmånad = rad2delar[3]
                                föddår    = rad2delar[4]
                            elif len(rad2delar) == 4:
                                föddmånad = rad2delar[2]
                                föddår    = rad2delar[3]
                            elif len(rad2delar) == 3:
                                föddår    = rad2delar[2]

                            Person1.född_år = str(föddår)

                            if föddmånad != "":
                                if månader.count(föddmånad) == 1:
                                    Person1.född_månad = str(månader.index(föddmånad)).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Född månad") # kan inte tolka månad

                            if födddag !="":
                                if födddag.isdigit() and 0<int(födddag)<32:
                                    Person1.född_dag = str(födddag).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Född dag") # kan inte tolka dag

                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3

                    elif rad2delar[1] == "PLAC":
                        pos = self.rader[2].index("PLAC")  # acceptera hela strängen efter PLAC, inkl white-space
                        källrad = self.rader[2][pos+4:]

                        Person1.född_disbytförsamling = källrad
                        if källrad[-1] == ')' and (källrad[-4:-2] == ' ('   or    källrad[-5:-3] == ' (' ):
                            Person1.född_församling = källrad[:-4]
                        else:
                            Person1.född_församling = källrad
                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3 för PLAC

                    elif rad2delar[1] == "SOUR":
                        pos = self.rader[2].index("SOUR")  # acceptera hela strängen efter SOUR, inkl white-space
                        källrad = self.rader[2][pos+4:]
                        Person1.född_källa = källrad
                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3 för SOUR

                    else:
                        self.skrivöverblivet(txtfil, self.rader, 2) # om annat än DATE, PLAC, SOUR på nivå 2
                else:
                    self.skrivöverblivet(txtfil, self.rader, 2) # ej förväntad längd på rad nivå 2
            else:
                self.skrivöverblivet(txtfil, self.rader, 1) # ej förväntad längd på rad nivå 1


# -------------------------------------------------
        elif  rad1delar[1] == "DEAT":

            if len(rad1delar) == 2:
                if len(rad2delar) > 2: # Om för kort rad så stupar man på nästa rad, om för lång: okänt innehåll

                    if rad2delar[1] == "DATE":
                        if len(rad2delar) > 5:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Död datum") # om mera än förväntat på DATE-raden
                        if len(rad2delar) < 3:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Död datum") # om mindre än förväntat på DATE-raden
                        else:
                            döddag   = ""
                            dödmånad = ""
                            dödar    = ""

                            if len(rad2delar) == 5:
                                döddag   = rad2delar[2]
                                dödmånad = rad2delar[3]
                                dödar    = rad2delar[4]
                            elif len(rad2delar) == 4:
                                dödmånad = rad2delar[2]
                                dödar    = rad2delar[3]
                            elif len(rad2delar) == 3:
                                dödar    = rad2delar[2]

                            Person1.död_år = str(dödar)

                            if dödmånad != "":
                                if månader.count(dödmånad) == 1:
                                    Person1.död_månad =str(månader.index(dödmånad)).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Död månad") # kan inte tolka månad

                            if döddag != "":
                                if döddag.isdigit() and 0<int(döddag)<32:
                                    Person1.död_dag = str(döddag).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Död dag") # kan inte tolka dag

                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3

                    elif rad2delar[1] == "PLAC":
                        pos = self.rader[2].index("PLAC")  # acceptera hela strängen efter PLAC, inkl white-space
                        källrad = self.rader[2][pos+4:]

                        Person1.död_disbytförsamling = källrad
                        if källrad[-1] == ')' and (källrad[-4:-2] == ' ('   or    källrad[-5:-3] == ' (' ):
                            Person1.död_församling = källrad[:-4]
                        else:
                            Person1.död_församling = källrad
                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3

                    elif rad2delar[1] == "SOUR":
                        pos = self.rader[2].index("SOUR")  # acceptera hela strängen efter SOUR, inkl white-space
                        källrad = self.rader[2][pos+4:]
                        Person1.död_källa = källrad
                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3 för SOUR

                    else:
                        self.skrivöverblivet(txtfil, self.rader, 2) # om annat än DATE, PLAC, SOUR på nivå 2
                else:
                    self.skrivöverblivet(txtfil, self.rader, 2) # ej förväntad längd på rad nivå 2
            else:
                self.skrivöverblivet(txtfil, self.rader, 1) # ej förväntad längd på rad nivå 1


# -------------------------------------------------
        elif  rad1delar[1] == "FAMS":
            if len(rad1delar) == 3:
                par_gedcom = rad1delar[2].strip('@')
                parindex = self.parlista_gedcom.index(par_gedcom)
                par_johan = self.parlista_johan[parindex]
                if Person1.egna_par_ID == "":
                    Person1.egna_par_ID = str(par_johan)
                else:
                    Person1.egna_par_ID = Person1.egna_par_ID + "\n" + str(par_johan)
            else:
                self.skrivöverblivet(txtfil, self.rader, 1)


# -------------------------------------------------
        elif  rad1delar[1] == "FAMC":
            if len(rad1delar) == 3:
                par_gedcom = rad1delar[2].strip('@')
                parindex = self.parlista_gedcom.index(par_gedcom)
                par_johan = self.parlista_johan[parindex]

                Person1.föräldrapar_ID = str(par_johan)
            else:
                self.skrivöverblivet(txtfil, self.rader, 1)


# -------------------------------------------------
        elif  rad1delar[1] == "CHAN":

            if len(rad1delar) == 2:
                if len(rad2delar) > 2: # Om för kort rad så stupar man på nästa rad, om för lång: okänt innehåll

                    if rad2delar[1] == "DATE":
                        if len(rad2delar) > 5:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Sparat datum") # om mera än förväntat på DATE-raden
                        elif len(rad2delar) <3:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Sparat datum") # om mindre än förväntat på DATE-raden
                        else:
                            sparatdag   = ""
                            sparatmånad = ""
                            sparatår    = ""

                            sparat_dag   = "  "
                            sparat_månad = "  "
                            sparat_tid = ""


                            if len(rad2delar) == 5:
                                sparatdag   = rad2delar[2]
                                sparatmånad = rad2delar[3]
                                sparatår    = rad2delar[4]
                            elif len(rad2delar) == 4:
                                sparatmånad = rad2delar[2]
                                sparatår    = rad2delar[3]
                            elif len(rad2delar) == 3:
                                sparatår    = rad2delar[2]

                            sparat_år = str(sparatår)

                            if sparatmånad != "":
                                if månader.count(sparatmånad) == 1:
                                    sparat_månad =str(månader.index(sparatmånad)).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Sparat månad") # kan inte tolka månad

                            if sparatdag != "":
                                if sparatdag.isdigit() and 0<int(sparatdag)<32:
                                    sparat_dag = str(sparatdag).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Sparat dag") # kan inte tolka dag


                            if len(rad3delar) > 2:
                                if rad3delar[1] == "TIME":
                                    pos = self.rader[3].index("TIME")

                                    sparat_tid = self.rader[3][pos+4:]
                                    self.skrivöverblivet(txtfil, self.rader, 4)  # ska inte finnas något alls på nivå 4 för TIME
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 3)  # om annat än TIME på nivå 3

                            sparat = sparat_år
                            if sparat_månad != "  ":
                                sparat = sparat + '-' + sparat_månad
                                if sparat_dag != "  ":
                                    sparat = sparat + '-' + sparat_dag + " " + sparat_tid

                            Person1.senastsparat = sparat
                    else:
                        self.skrivöverblivet(txtfil, self.rader, 2)  # om annat än DATE på nivå 2
                else:
                    self.skrivöverblivet(txtfil, self.rader, 2) # ej förväntad längd på rad nivå 2
            else:
                self.skrivöverblivet(txtfil, self.rader, 1) # ej förväntad längd på rad nivå 1




# -------------------------------------------------
        else:
            self.skrivöverblivet(txtfil, self.rader, 1) # om annat än NAME, SEX, BIRT, DEAT, FAMS, FAMC, CHAN på nivå 1



        txtfil.close()
        allmänt.sparapersonfil(allmänt(), xmlfilnamn, Person1)

        return
###################################################################################################




###################################################################################################
    def skrivöverblivet(self, txtfil, rader, startnummer, text =""):
        for rad in rader[startnummer:]:
            if rad== '*':
                break
            if text=="":
                txtfil.write(rad  + '\n' )
            else:
                txtfil.write(text + ":  " + rad  + '\n' )
        return
###################################################################################################




###################################################################################################
    def parblad(self):

        månader = ['   ', 'JAN','FEB','MAR','APR','MAY','JUN','JUL','AUG','SEP','OCT','NOV','DEC']

        rad0delar = self.rader[0].strip().split(' ')
        rad1delar = self.rader[1].strip().split(' ')
        rad2delar = self.rader[2].strip().split(' ')
        rad3delar = self.rader[3].strip().split(' ')
#        rad4delar = self.rader[4].strip().split(' ')
#        rad5delar = self.rader[5].strip().split(' ')
#        rad6delar = self.rader[6].strip().split(' ')
#        rad7delar = self.rader[7].strip().split(' ')
#        rad8delar = self.rader[8].strip().split(' ')
#        rad9delar = self.rader[9].strip().split(' ')


        if rad1delar[0] == "*":
            return
        if len(rad1delar) < 2 :
            return

        par_gedcom = rad0delar[1].strip('@')
        parindex = self.parlista_gedcom.index(par_gedcom)
        par_johan = self.parlista_johan[parindex]


        if self.senastesignal != par_johan:
            self.emit(SIGNAL("indexed(QString)"), par_johan)
            self.senastesignal = par_johan



        underkatalog = os.path.join(self.nydatabas, "data", par_johan[0:8])
        if not os.path.exists(underkatalog):
            os.mkdir(underkatalog)
        xmlfilnamn = os.path.join(underkatalog, par_johan + ".xml")
        txtfilnamn = os.path.join(underkatalog, par_johan + ".txt")



        txtfil = open(txtfilnamn, 'a')

        Par1 = johanparfil()
        allmänt.initparfil(allmänt(), Par1)

        if not os.path.exists(xmlfilnamn):
            Par1.par_ID = str(par_johan)
            Par1.anteckning =  'FAM-nr ' + par_gedcom + ', från gedcom-fil ' + self.importfil
            allmänt.sparaparfil(allmänt(), xmlfilnamn, Par1)

        allmänt.hämtaparfil(allmänt(), xmlfilnamn, Par1)

        if rad1delar[1] == "HUSB":
            if len(rad1delar) == 3:
                person_gedcom = rad1delar[2].strip('@')
                personindex = self.personlista_gedcom.index(person_gedcom)
                person_johan = self.personlista_johan[personindex]
                Par1.person1_ID = person_johan
            else:
                self.skrivöverblivet(txtfil, self.rader, 1)
#                txtfil.write( self.rader[1] + '\n' )

        elif rad1delar[1] == "WIFE":
            if len(rad1delar) == 3:
                person_gedcom = rad1delar[2].strip('@')
                personindex = self.personlista_gedcom.index(person_gedcom)
                person_johan = self.personlista_johan[personindex]
                Par1.person2_ID = person_johan
            else:
                self.skrivöverblivet(txtfil, self.rader, 1)
#                txtfil.write( self.rader[1]  + '\n')

        elif rad1delar[1] == "CHIL":

            if len(rad1delar) == 3:
                person_gedcom = rad1delar[2].strip('@')
                personindex = self.personlista_gedcom.index(person_gedcom)
                person_johan = self.personlista_johan[personindex]

                if Par1.barn_ID == "":
                    Par1.barn_ID = person_johan
                else:
                    Par1.barn_ID = Par1.barn_ID + '\n' + person_johan
            else:
                self.skrivöverblivet(txtfil, self.rader, 1)
#                txtfil.write( self.rader[1] + '\n' )

 # -------------------------------------------------
        elif rad1delar[1] == "MARR":

            if len(rad1delar) == 2:
                if len(rad2delar) > 2: # Om för kort rad så stupar man på nästa rad, om för lång: okänt innehåll

                    if rad2delar[1] == "DATE":
                        if len(rad2delar) > 5:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Inledning datum") # om mera än förväntat på DATE-raden
                        elif len(rad2delar) < 3:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Inledning datum") # om mindre än förväntat på DATE-raden
                        else:
                            inledndag   = ""
                            inlednmånad = ""
                            inlednår    = ""
                            if len(rad2delar) == 5:
                                inledndag   = rad2delar[2]
                                inlednmånad = rad2delar[3]
                                inlednår    = rad2delar[4]
                            elif len(rad2delar) == 4:
                                inlednmånad = rad2delar[2]
                                inlednår    = rad2delar[3]
                            elif len(rad2delar) == 3:
                                inlednår    = rad2delar[2]

                            Par1.inledn_år = str(inlednår)

                            if inlednmånad != "":
                                if månader.count(inlednmånad) == 1:
                                    Par1.inledn_månad =str(månader.index(inlednmånad)).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Inledning månad") # kan inte tolka månad

                            if inledndag !="":
                                if inledndag.isdigit() and 0<int(inledndag)<32:
                                    Par1.inledn_dag = str(inledndag).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Inledning dag") # kan inte tolka dag

                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3

                    elif rad2delar[1] == "PLAC":
                        pos = self.rader[2].index("PLAC")  # acceptera hela strängen efter PLAC, inkl white-space
                        källrad = self.rader[2][pos+4:]

                        Par1.inledn_disbytförsamling = källrad
                        if källrad[-1] == ')' and (källrad[-4:-2] == ' ('   or    källrad[-5:-3] == ' (' ):
                            Par1.inledn_församling = källrad[:-4]
                        else:
                            Par1.inledn_församling = källrad
                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3


                    elif rad2delar[1] == "SOUR":
                        pos = self.rader[2].index("SOUR")  # acceptera hela strängen efter SOUR, inkl white-space
                        källrad = self.rader[2][pos+4:]
                        Par1.inledn_källa = källrad
                        self.skrivöverblivet(txtfil, self.rader, 3)  # ska inte finnas något alls på nivå 3

                    else:
                        self.skrivöverblivet(txtfil, self.rader, 2) # om annat än DATE, PLAC, SOUR på nivå 2
                else:
                    self.skrivöverblivet(txtfil, self.rader, 2) # ej förväntad längd på rad nivå 2
            else:
                self.skrivöverblivet(txtfil, self.rader, 1) # ej förväntad längd på rad nivå 1


# -------------------------------------------------
        elif  rad1delar[1] == "CHAN":

            if len(rad1delar) == 2:
                if len(rad2delar) > 2: # Om för kort rad så stupar man på nästa rad, om för lång: okänt innehåll

                    if rad2delar[1] == "DATE":
                        if len(rad2delar) > 5:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Sparat datum") # om mera än förväntat på DATE-raden
                        elif len(rad2delar) <3:
                            self.skrivöverblivet(txtfil, self.rader, 2, "Sparat datum") # om mindre än förväntat på DATE-raden
                        else:
                            sparatdag   = ""
                            sparatmånad = ""
                            sparatår    = ""

                            sparat_dag   = "  "
                            sparat_månad = "  "
                            sparat_tid = ""


                            if len(rad2delar) == 5:
                                sparatdag   = rad2delar[2]
                                sparatmånad = rad2delar[3]
                                sparatår    = rad2delar[4]
                            elif len(rad2delar) == 4:
                                sparatmånad = rad2delar[2]
                                sparatår    = rad2delar[3]
                            elif len(rad2delar) == 3:
                                sparatår    = rad2delar[2]

                            sparat_år = str(sparatår)

                            if sparatmånad != "":
                                if månader.count(sparatmånad) == 1:
                                    sparat_månad =str(månader.index(sparatmånad)).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Sparat månad") # kan inte tolka månad

                            if sparatdag != "":
                                if sparatdag.isdigit() and 0<int(sparatdag)<32:
                                    sparat_dag = str(sparatdag).zfill(2)
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 2, "Sparat dag") # kan inte tolka dag


                            if len(rad3delar) > 2:
                                if rad3delar[1] == "TIME":
                                    pos = self.rader[3].index("TIME")
                                    sparat_tid = self.rader[3][pos+4:]

                                    self.skrivöverblivet(txtfil, self.rader, 4)  # ska inte finnas något alls på nivå 4 för TIME
                                else:
                                    self.skrivöverblivet(txtfil, self.rader, 3)  # om annat än TIME på nivå 3

                            sparat = sparat_år
                            if sparat_månad != "  ":
                                sparat = sparat + '-' + sparat_månad
                                if sparat_dag != "  ":
                                    sparat = sparat + '-' + sparat_dag + " " + sparat_tid

                            Par1.senastsparat = sparat
                    else:
                        self.skrivöverblivet(txtfil, self.rader, 2)  # om annat än DATE på nivå 2
                else:
                    self.skrivöverblivet(txtfil, self.rader, 2) # ej förväntad längd på rad nivå 2
            else:
                self.skrivöverblivet(txtfil, self.rader, 1) # ej förväntad längd på rad nivå 1


# -------------------------------------------------
        else:
            self.skrivöverblivet(txtfil, self.rader, 1) # om annat än HUSB, WIFE, CHIL, MARR, CHAN på nivå 1

        txtfil.close()
        allmänt.sparaparfil(allmänt(), xmlfilnamn, Par1)

        return
###################################################################################################




###################################################################################################
    def adderanamn(self, namn, unam):
        splitnamn = namn.split("\n")

        if namn == "":
            return unam

        if unam in splitnamn:
            return namn
        else:
            return namn + "\n" + unam
###################################################################################################





###################################################################################################
    def gemignästarad(self):   # - Ge mig nästa rad
#
# Förutsättning: filen som läses har kodning utf-8.
# Den ska vara öppnad med globala namnet gedcomfil
#
# Alla leveranser till anropande kod sker i utf-8 i listan self.rader[]
#
# Anropas från två ställen ovan, i avsnittet "fyll i johandatabasen" i (stora) rutinen "importera"

        ändrad = False
        if  self.redanlästrad != "*"  :
            rad = self.redanlästrad
            self.redanlästrad = "*"
        else:
            rårad = self.gedcomfil.readline()
            if rårad == '':
                self.EndOfFile = True
                return
            rad = rårad.strip()

        delar = rad.split(' ')

        if len(delar) > 1:
            if delar[0].isdigit():

                nivå = int(delar[0])
                self.rader[nivå] = rad
                self.rader[nivå+1] = "*"
                self.senastenivå = nivå
                ändrad = True

        while True:
            rårad = self.gedcomfil.readline()
            if rad == '':
                self.EndOfFile = True
                return
            rad = rårad.strip()
            delar = rad.split(' ')


            if len(delar) > 1:
                if delar[0].isdigit():
                    nivå = int(delar[0])
                    if (self.senastenivå +1 - nivå != 0  ):
                        self.redanlästrad = rad        # spara till nästa gång
                        return

                    else:
                        self.rader[nivå] = rad
                        self.senastenivå = nivå
                        if nivå+3 >= len(self.rader):
                            self.rader.append("*")
                        self.rader[nivå+1] = "*"

        return
###################################################################################################




#############################################################################################
    def logga(self, txt):
        import datetime     # behöver importeras här också för att rutinen ska fungera under stängning, anropad från __del__
# logga till loggfil
        if self.loggfilfolder != "":

            if not os.path.exists(self.loggfilfolder):
                os.mkdir(self.loggfilfolder)

            if os.path.exists(self.loggfilfolder):

                if self.loggfilopen == False:
                    dt = datetime.datetime.today()
                    self.loggfilnamn = os.path.join(self.loggfilfolder, self.loggfilgrundnamn + "_" + dt.strftime('%Y%m%d_%H%M%S') +'.log')
                    self.loggfil = open(self.loggfilnamn,'w', encoding='utf-8')
                    self.loggfilopen = True

                    self.logga ('Startar logg för ImporteraGedcom_py3\n')

                dt = datetime.datetime.today()
                msg =  str(dt.strftime("%Y-%m-%d_%H:%M:%S")) + '.' + str(int(dt.microsecond/1000)).zfill(3) + '   ' + txt
                self.loggfil.write (msg)

## logga till konsolfönstret
#        if txt[-1:] == '\n':
#            ptxt=txt[:-1]
#        else:
#            ptxt = txt + ','
#        print(ptxt)

## logga tilltextruta i johanfönstret
#        self.textEdit_logg.insertPlainText(txt )
#        self.textEdit_logg.moveCursor(QtGui.QTextCursor.End, QtGui.QTextCursor.MoveAnchor)
#
#        textEdit_logg_Rect = self.textEdit_logg.geometry()   # QtCore.QRect ( )

## gör detta någon nytta?
#        self.textEdit_logg.repaint( textEdit_logg_Rect )
#

## gör detta någon nytta?
#        self.repaint()
#

## gör detta någon nytta?
#        app.processEvents()

#############################################################################################




###################################################################################################
    def avsluta(self, exitcode=0):

        if self.loggfilopen == True:
            self.logga ('Avslutar logg för ImporteraGedcom_py3\n')
            self.loggfilopen = False
            self.loggfil.close()

###################################################################################################
